-- DummyMac.applescript
-- DummyMac

--  Created by hany on 27/09/05.
--  Copyright 2005 hanynet.com. All rights reserved. 
--  DummyMac is a frontend for the traffic shaper module DUMMYNET, part of ipfw2, bundled in Mac OS X 10.4.


-- PROPERTIES
property objectName : missing value
property pipeActive : missing value
property srcPort : missing value
property dstPort : missing value
property pipeBand : missing value
property pipeScale : missing value
property delayMS : missing value
property ruleExists : missing value

-- CLICKED handler
on clicked theObject
	-- watch ipfw rules
	if the name of theObject is "watch" then
		imBusy()
		set the contents of text field "title" of window "list" to "Current IPFW rules:"
		set the contents of text view "list" of scroll view "list" of window "list" to do shell script "sudo ipfw list" with administrator privileges
		display window "list" attached to window "main"
		-- delay
	else if the name of theObject is "delay" then
		set enabled of text field "delay" of box "box" of window "main" to the contents of button "delay" of box "box" of window "main" as boolean
		-- watch ipfw pipes
	else if the name of theObject is "watch2" then
		imBusy()
		set the contents of text field "title" of window "list" to "Current IPFW pipes:"
		set the contents of text view "list" of scroll view "list" of window "list" to do shell script "sudo ipfw pipe list" with administrator privileges
		display window "list" attached to window "main"
		-- generic close panel
	else if the name of theObject is "closepanel" then
		set the contents of text view "list" of scroll view "list" of window "list" to ""
		endBusy()
		close panel (window of theObject)
		-- help
	else if the name of theObject is "help" then
		display window "help" attached to window "main"
	else
		-- ENABLE DUMMYMAC / DISABLE DUMMYMAC
		imBusy()
		set objectName to the name of theObject as string
		if the contents of button objectName of box "box" of window "main" is equal to "True" as boolean then
			set dstPort to the contents of text field "customports" of box "box" of window "main"
			set pipeBand to the contents of text field objectName of box "box" of window "main"
			-- set scale
			if the current row of matrix "scale" of box "box" of window "main" is equal to "1" as number then
				set pipeScale to "Kbit/s"
			else
				set pipeScale to "Mbit/s"
			end if
			-- set direction
			if the current row of matrix "dir" of box "box" of window "main" is equal to "1" as number then
				set srcPort to dstPort
				set dstPort to ""
			else if the current row of matrix "dir" of box "box" of window "main" is equal to "2" as number then
				set srcPort to ""
			else
				set srcPort to dstPort
			end if
			-- set delay
			if contents of button "delay" of box "box" of window "main" is equal to "True" as boolean then
				set delayTime to the contents of text field "delay" of box "box" of window "main"
				set delayMS to " delay " & delayTime & "ms"
			else
				set delayMS to ""
			end if
			createPipe()
			hideItems()
			endBusy()
		else
			imBusy()
			killPipe()
			showItems()
			endBusy()
		end if
		
	end if
end clicked

-- FUNCTION (create pipe with IPFW)
on createPipe()
	checkPipe()
	try
		do shell script "sudo ipfw add 111 pipe 111 ip from any " & srcPort & " to any " & dstPort with administrator privileges
		do shell script "sudo ipfw pipe 111 config bw " & pipeBand & pipeScale & delayMS with administrator privileges
		set pipeActive to 1
	on error errorText
		display dialog ("DummyMac Error: " & errorText) with icon 1
		set the contents of button objectName of box "box" of window "main" to "False" as boolean
		hideItems()
	end try
end createPipe

-- FUNCTION (delete IPFW rule and flush pipe)
on killPipe()
	do shell script "sudo ipfw del 111" with administrator privileges
	(* do shell script "sudo ipfw pipe 111 del" with administrator privileges (KERNEL PANIC... but we don't really need to delete pipe settings.) *)
	set pipeActive to 0
end killPipe

-- FUNCTION (check existing ipfw rule number 111)
on checkPipe()
	set ruleExists to do shell script "sudo ipfw list | grep 111 | cut -d \" \" -f 1" with administrator privileges
	if ruleExists is equal to "00111" as string then
		set oldRule to do shell script "sudo ipfw list | grep 111" with administrator privileges
		do shell script "sudo ipfw del 111" with administrator privileges
		set the contents of text field "title" of window "list" to "Warning, old IPFW rule deleted"
		set the contents of text view "list" of scroll view "list" of window "list" to "There was a conflict with old ipfw rule number 111. 
 Old rule was: 
  " & oldRule & "
The rule was deleted and will be replaced with the new one."
		display window "list" attached to window "main"
	end if
end checkPipe

-- FUNCTION (make GUI enabled)
on showItems()
	tell box "box" of window "main"
		set enabled of slider objectName to true
		set enabled of text field objectName to true
		set enabled of text field "customports" of box "box" of window "main" to true
		set enabled of matrix "scale" of box "box" of window "main" to true
		set enabled of matrix "dir" of box "box" of window "main" to true
		set enabled of text field "delay" of box "box" of window "main" to the contents of button "delay" of box "box" of window "main" as boolean
		set enabled of button "delay" of box "box" of window "main" to true
		set enabled of popup button 1 of box "box" of window "main" to true
	end tell
end showItems

-- FUNCTION (make GUI disabled)
on hideItems()
	tell box "box" of window "main"
		set enabled of slider objectName to false
		set enabled of text field objectName to false
		set enabled of text field "customports" of box "box" of window "main" to false
		set enabled of matrix "scale" of box "box" of window "main" to false
		set enabled of matrix "dir" of box "box" of window "main" to false
		set enabled of text field "delay" of box "box" of window "main" to false
		set enabled of button "delay" of box "box" of window "main" to false
		set enabled of popup button 1 of box "box" of window "main" to false
	end tell
end hideItems

-- FUNCTION (get SLIDER value)
on action theObject
	set objectName to the name of theObject
	tell box "box" of window "main"
		set the contents of text field objectName to the contents of slider objectName as integer
	end tell
end action

-- FUNCTION (set SLIDER value from text field)
on end editing theObject
	if the name of theObject is "dummy" then
		set the contents of slider "dummy" of box "box" of window "main" to the contents of text field "dummy" of box "box" of window "main"
		
	end if
end end editing

-- FUNCTION (quit application if window is closed)
on should quit after last window closed theObject
	try
		do shell script "sudo ipfw del 111" with administrator privileges
	on error errorText
	end try
	quit
end should quit after last window closed

-- FUNCTION (choose menu)
on choose menu item theObject
	if the name of theObject is "about" then
		display window "about"
	else if the name of theObject is equal to "adviceme" then
		set rangeName to the title of popup button 1 of box "box" of window "main"
		if rangeName = "web" then
			set the contents of text field "customports" of box "box" of window "main" to "80"
		else if rangeName is equal to "ftp" then
			set the contents of text field "customports" of box "box" of window "main" to "20,21"
		else if rangeName is equal to "web+pop3" then
			set the contents of text field "customports" of box "box" of window "main" to "80,3128,110"
		else if rangeName is equal to "ichat" then
			set the contents of text field "customports" of box "box" of window "main" to "4020,5190,16384-16403"
		else if rangeName is equal to "pop3" then
			set the contents of text field "customports" of box "box" of window "main" to "110"
		else if rangeName is equal to "bittorrent default" then
			set the contents of text field "customports" of box "box" of window "main" to "6881-6900"
		else if rangeName is equal to "bittorrent+high ports" then
			set the contents of text field "customports" of box "box" of window "main" to "6881-65535"
		else if rangeName is equal to "unpriv. ports" then
			set the contents of text field "customports" of box "box" of window "main" to "1025-65535"
		else if rangeName is equal to "priv. ports" then
			set the contents of text field "customports" of box "box" of window "main" to "1-1024"
		else if rangeName is equal to "all" then
			set the contents of text field "customports" of box "box" of window "main" to "1-65535"
		end if
		set the contents of text field "debug" of window "main" to (rangeName)
	end if
end choose menu item

-- FUNCTION (busy progress indicator)
on imBusy()
	set visible of progress indicator "busy" of window "main" to true
	tell progress indicator "busy" of window "main" to start
end imBusy
on endBusy()
	set visible of progress indicator "busy" of window "main" to false
	tell progress indicator "busy" of window "main" to stop
end endBusy


















